@@ -426,7 +426,8 @@ static gboolean cb_ss_control(int type, const char *sc,
 		return TRUE;
 	}
 
-	/* According to 27.007, AG, AC and AB only work with mode = 0
+	/*
+	 * According to 27.007, AG, AC and AB only work with mode = 0
 	 * We support query by querying all relevant types, since we must
 	 * do this for the deactivation case anyway
 	 */
@@ -580,6 +581,11 @@ static void cb_unregister_ss_controls(struct ofono_call_barring *cb)
 	__ofono_ussd_passwd_unregister(cb->ussd, "353");
 }
 
+gboolean __ofono_call_barring_is_busy(struct ofono_call_barring *cb)
+{
+	return cb->pending ? TRUE : FALSE;
+}
+
 static inline void cb_append_property(struct ofono_call_barring *cb,
 					DBusMessageIter *dict, int start,
 					int end, int cls, const char *property)
@@ -669,7 +675,7 @@ static DBusMessage *cb_get_properties(DBusConnection *conn, DBusMessage *msg,
 {
 	struct ofono_call_barring *cb = data;
 
-	if (cb->pending)
+	if (cb->pending || __ofono_ussd_is_busy(cb->ussd))
 		return __ofono_error_busy(msg);
 
 	if (!cb->driver->query)
@@ -733,7 +739,8 @@ static void set_lock_callback(const struct ofono_error *error, void *data)
 		return;
 	}
 
-	/* If we successfully set the value, we must query it back
+	/*
+	 * If we successfully set the value, we must query it back
 	 * Call Barring is a special case, since according to 22.088 2.2.1:
 	 * "The PLMN will ensure that only one of the barring programs is
 	 * active per basic service group. The activation of one specific
@@ -780,7 +787,8 @@ static gboolean cb_lock_property_lookup(const char *property, const char *value,
 		return FALSE;
 	}
 
-	/* Gah, this is a special case.  If we're setting a barring to
+	/*
+	 * Gah, this is a special case.  If we're setting a barring to
 	 * disabled, then generate a disable all outgoing/incoming
 	 * request for a particular basic service
 	 */
@@ -822,7 +830,7 @@ static DBusMessage *cb_set_property(DBusConnection *conn, DBusMessage *msg,
 	int cls;
 	int mode;
 
-	if (cb->pending)
+	if (cb->pending || __ofono_ussd_is_busy(cb->ussd))
 		return __ofono_error_busy(msg);
 
 	if (!dbus_message_iter_init(msg, &iter))
@@ -894,7 +902,7 @@ static DBusMessage *cb_disable_all(DBusConnection *conn, DBusMessage *msg,
 	if (!cb->driver->set)
 		return __ofono_error_not_implemented(msg);
 
-	if (cb->pending)
+	if (cb->pending || __ofono_ussd_is_busy(cb->ussd))
 		return __ofono_error_busy(msg);
 
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &passwd,
@@ -941,7 +949,7 @@ static DBusMessage *cb_set_passwd(DBusConnection *conn, DBusMessage *msg,
 	if (!cb->driver->set_passwd)
 		return __ofono_error_not_implemented(msg);
 
-	if (cb->pending)
+	if (cb->pending || __ofono_ussd_is_busy(cb->ussd))
 		return __ofono_error_busy(msg);
 
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &old_passwd,
